/* 
 *
 *            Copyright (C) 1999 Hewlett-Packard Company
 *                       ALL RIGHTS RESERVED.
 *
 * The enclosed software and documentation includes copyrighted works
 * of Hewlett-Packard Co. For as long as you comply with the following
 * limitations, you are hereby authorized to (i) use, reproduce, and
 * modify the software and documentation, and to (ii) distribute the
 * software and documentation, including modifications, for
 * non-commercial purposes only.
 *      
 * 1.  The enclosed software and documentation is made available at no
 *     charge in order to advance the general development of
 *     high-performance networking products.
 * 
 * 2.  You may not delete any copyright notices contained in the
 *     software or documentation. All hard copies, and copies in
 *     source code or object code form, of the software or
 *     documentation (including modifications) must contain at least
 *     one of the copyright notices.
 * 
 * 3.  The enclosed software and documentation has not been subjected
 *     to testing and quality control and is not a Hewlett-Packard Co.
 *     product. At a future time, Hewlett-Packard Co. may or may not
 *     offer a version of the software and documentation as a product.
 * 
 * 4.  THE SOFTWARE AND DOCUMENTATION IS PROVIDED "AS IS".
 *     HEWLETT-PACKARD COMPANY DOES NOT WARRANT THAT THE USE,
 *     REPRODUCTION, MODIFICATION OR DISTRIBUTION OF THE SOFTWARE OR
 *     DOCUMENTATION WILL NOT INFRINGE A THIRD PARTY'S INTELLECTUAL
 *     PROPERTY RIGHTS. HP DOES NOT WARRANT THAT THE SOFTWARE OR
 *     DOCUMENTATION IS ERROR FREE. HP DISCLAIMS ALL WARRANTIES,
 *     EXPRESS AND IMPLIED, WITH REGARD TO THE SOFTWARE AND THE
 *     DOCUMENTATION. HP SPECIFICALLY DISCLAIMS ALL WARRANTIES OF
 *     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * 5.  HEWLETT-PACKARD COMPANY WILL NOT IN ANY EVENT BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
 *     (INCLUDING LOST PROFITS) RELATED TO ANY USE, REPRODUCTION,
 *     MODIFICATION, OR DISTRIBUTION OF THE SOFTWARE OR DOCUMENTATION.
 */

/*
 * Copyright (c) 1998 Luigi Rizzo
 *
 * Redistribution and use in source forms, with and without modification,
 * are permitted provided that this entire comment appears intact.
 *
 * Redistribution in binary form may occur without any restrictions.
 * Obviously, it would be nice if you gave credit where credit is due
 * but requiring it would be too onerous.
 *
 * This software is provided ``AS IS'' without any warranties of any kind.
 *
 *	$Id: ip_dummynet.h,v 1.3 1999/01/23 23:59:50 archie Exp $
 */

#ifndef _IP_DUMMYNET_H
#define _IP_DUMMYNET_H


/*
 * Streams module ID for dummynet.  knuth only knows if this conflicts.
 */
#define DUMMYNET_ID 56821

/*
 * dummynet data structures
 *
 *    dn_d	Dummynet desciptor.  Consists of two pipes: one for outbound
 *              and the other for inbound.
 *    dn_pipe	Pipe descriptor.  Containts parameters for simulation:
 *		bandwidth, delay, frame error rate, etc.  Also, it contains
 *		two queues: router queue and pipe queue.
 *
 *    Dummynet has a pair of pipes: one for outbound (write-side,
 *    sending-side), and one for inbound (read-side, receiving-side).
 *    Each pipe has a router queue and a pipe queue.  The router
 *    queue controls the bandwidth of the link simulated by the
 *    dummynet.  The pipe queue controls the latency of th link
 *    simulated by the dummynet.  Each packet is first queued in
 *    the router queue, and waits for the sufficient bandwidth
 *    become available for sending the packet.  Then, the packet
 *    is moved to the pipe queue, and waits for the delay/latency
 *    time to be passed.  The delay time simulates the latency
 *    between the sender and receiver over the simulated link,
 *    and it includes the processing time of the transmiter and
 *    the receiver, the propagation delay, the processing time
 *    of link-levle frame retransmission(s), etc.
 *
 *    Note:  I don't use dn_pkt any more.  dn_pkt used to be used to store 
 *    a pointer to the next packet, packet length, and packet delay.  Instead,
 *    I just use msgb's b_next, b_quad[0], and b_quad[1] respectively.
 *    This makes the code a little less portable to other STREAMS environment.
 *    But, now we don't have to allocate packet headers, don't have to handle
 *    memory allocation errors, and so on.
 */
#define DN_PKT_LEN(mp)		(&(mp)->b_quad[0])
#define DN_PKT_XMIT_LEN(mp)	(&(mp)->b_quad[1])
#define DN_PKT_DELAY(mp)	(&(mp)->b_quad[2])
#define DN_PKT_ERROR(mp)	(&(mp)->b_quad[3])

/*
 * Dummynet pipe struct.
 */
struct dn_pipe {
	unsigned int pipe_flags;
	int	pipe_id ;	    /* id		*/
	queue_t	*owner_q;	    /* associated streams queue */
	MBLKP	routerq_head;	    /* the "router" queue	*/
	MBLKP	routerq_tail;
	int	routerq_len;	    /* elements in router queue */
	int	routerq_len_bytes;  /* bytes in router queue */
	int	routerq_size;	    /* # of packets: 1st choice	*/
	int	routerq_size_bytes; /* bytes:        2nd choice */
	int	routerq_drops;	    /* drops from r_queue */
	int     router_q_hiwat;	    /* high water mark of router queue */
	int	bandwidth_bps;      /* bits per second	*/
	long	numbytes;	    /* available bandwidth */
	long	bytes_borrowed;  /* bytes borrowed from next tick's bandwidth*/
	MBLKP	pipeq_head;	    /* the "pipe" queue	*/
	MBLKP	pipeq_tail;	
	int	delay_ticks;	    /* delay ticks */
	int   	frame_payload_size; /* frame's payload size (bytes) */
	int	ferror_per_million; /* # frame errors per million frames */
	int	ferror_delay_ticks; /* cost of local retransmit in ticks*/
	int	ferror_rxmit_trials;/* # of attempts of local retransmits 
		                     * excluding the original transmit */
};
typedef struct dn_pipe	dn_pipe_t;
/*
 * pipe_flags
 */
#define DN_PIPE_ENABLED		0x01 /* dummynet pipe processing is enabled */

/*
 * Dummynet per-instance private data struct.
 */
struct dn_d {
	unsigned int	dn_flags;
	clock_t		dn_last_ticks;    /* Last time timeout routine ran. */
	mblk_t	 	*dn_timer_mp;     /* dummy mblk used for streams_put */
	toid_t 	 	dn_timer_id;      /* outstanding timeout's id */
	dn_pipe_t    	dn_pipe_outbound; /* outbound pipe, write-side */
	dn_pipe_t    	dn_pipe_inbound;  /* inbound pipe, read-side */
};
typedef struct dn_d	dn_d_t;

/*
 * dn_flags
 */
#define DN_CLOSING    		  0x01 /* dummynet module is closing */
#define	DN_BETWEEN_IP_AND_DRIVER  0x02 /* dummynet is between driver and ip.*/
#define DN_BETWEEN_TCP_UDP_AND_IP 0x04 /* dummynet is between tcp/udp and ip */

#endif /* _IP_DUMMYNET_H */
